import { QueryInterpolation, TaggedType } from "./wire-protocol";
import { QueryArgument } from "./query-builder";
export interface DecodeOptions {
    long_type: "number" | "bigint";
}
/**
 * TaggedType provides the encoding/decoding of the Fauna Tagged Type formatting
 */
export declare class TaggedTypeFormat {
    /**
     * Encode the value to the Tagged Type format for Fauna
     *
     * @param input - value that will be encoded
     * @returns Map of result
     */
    static encode(input: QueryArgument): TaggedType;
    /**
     * Encode the value to a QueryInterpolation to send to Fauna
     *
     * @param input - value that will be encoded
     * @returns Map of result
     */
    static encodeInterpolation(input: QueryArgument): QueryInterpolation;
    /**
     * Decode the JSON string result from Fauna to remove Tagged Type formatting.
     *
     * @param input - JSON string result from Fauna
     * @returns object of result of FQL query
     */
    static decode(input: string, decodeOptions: DecodeOptions): any;
}
export declare const LONG_MIN: bigint;
export declare const LONG_MAX: bigint;
export declare const INT_MIN: number;
export declare const INT_MAX: number;
